package cn.com.qmth.examcloud.task.base.multithread;

import org.apache.logging.log4j.ThreadContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class Consumer<T>  extends Thread{
	private static final Logger LOG = LoggerFactory.getLogger(Consumer.class);
	private Basket basket;
	
	private String traceId;
	
	public Consumer() {
	}
	@Override
	public void run() {
		ThreadContext.put("TRACE_ID", traceId);
		LOG.info("*******************Consumer:"+Thread.currentThread().getId()+" start");
		try {
			while (true) {
				//先判断是否有异常结束
				if(basket.isExcuteError()) {
					break;
				}
				//取消费数据
				Object o= basket.consume();
				//判断消费数据是否是结束
				if(o instanceof EndObject) {
					break;
				}
				@SuppressWarnings("unchecked")
				T t=(T)o;
				LOG.info("*******************Consumer:"+Thread.currentThread().getId()+" consume");
				//消费数据实现
				consume(t);
			}
		} catch (Exception e) {
			basket.setExcuteError(true);
			LOG.error("消费线程处理出错",e);
		}finally {
			basket.countDown();
			LOG.info("*******************Consumer:"+Thread.currentThread().getId()+" stop");
			ThreadContext.clearAll();
		}
	}
	public abstract void consume(T t);
	public Basket getBasket() {
		return basket;
	}
	public void setBasket(Basket basket) {
		this.basket = basket;
	}
	public String getTraceId() {
		return traceId;
	}
	public void setTraceId(String traceId) {
		this.traceId = traceId;
	}
	
}
