package cn.com.qmth.examcloud.task.service.consumer;

import java.util.ArrayList;
import java.util.List;

import cn.com.qmth.examcloud.core.questions.api.ExtractConfigCloudService;
import cn.com.qmth.examcloud.core.questions.api.request.GetBasePaperReq;
import cn.com.qmth.examcloud.core.questions.api.response.GetBasePaperResp;
import cn.com.qmth.examcloud.marking.api.MarkItemCloudService;
import cn.com.qmth.examcloud.marking.api.bean.MarkItemBean;
import cn.com.qmth.examcloud.marking.api.request.SaveMarkItemReq;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultPaper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionGroup;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionStructureWrapper;
import cn.com.qmth.examcloud.question.commons.core.paper.DefaultQuestionUnitWrapper;
import cn.com.qmth.examcloud.task.base.multithread.Consumer;
import cn.com.qmth.examcloud.task.service.dto.MarkWorkCreateItemsDto;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

public class MarkWorkCreateItemsConsumer extends Consumer<MarkWorkCreateItemsDto> {

	private ExtractConfigCloudService extractConfigCloudService = SpringContextHolder
			.getBean(ExtractConfigCloudService.class);

	private MarkItemCloudService markItemCloudService = SpringContextHolder.getBean(MarkItemCloudService.class);
	private static final double SCORE_INTERVAL = 0.5;

	@Override
	public void consume(MarkWorkCreateItemsDto dto) {
		List<String> basePaperIds = dto.getList();
		Long workId = dto.getWorkId();
		List<MarkItemBean> markItems = new ArrayList<MarkItemBean>();
		GetBasePaperReq getBasePaperReq = new GetBasePaperReq();
		for (String basePaperId : basePaperIds) {
			getBasePaperReq.setPaperId(basePaperId);
			GetBasePaperResp getBasePaperResp = extractConfigCloudService.getBasePaper(getBasePaperReq);
			DefaultPaper defaultPaper = getBasePaperResp.getDefaultPaper();
			List<DefaultQuestionGroup> defaultQuestionGroupList = defaultPaper.getQuestionGroupList();
			// 大题号
			int mainNumber = 1;
			// 每个小题在试卷中的位置
			int order = 0;
			for (DefaultQuestionGroup defaultQuestionGroup : defaultQuestionGroupList) {
				List<DefaultQuestionStructureWrapper> defaultQuestionStructureWrapperList = defaultQuestionGroup
						.getQuestionWrapperList();
				// 小题号
				int subNumber = 1;
				for (DefaultQuestionStructureWrapper defaultQuestionStructureWrapper : defaultQuestionStructureWrapperList) {
					List<DefaultQuestionUnitWrapper> defaultQuestionUnitWrapperList = defaultQuestionStructureWrapper
							.getQuestionUnitWrapperList();
					for (DefaultQuestionUnitWrapper defaultQuestionUnitWrapper : defaultQuestionUnitWrapperList) {
						order++;
						if ("FILL_UP".equals(defaultQuestionUnitWrapper.getQuestionType().toString())
								|| "ESSAY".equals(defaultQuestionUnitWrapper.getQuestionType().toString())) {
							MarkItemBean markItem = new MarkItemBean(basePaperId, mainNumber,
									subNumber, defaultQuestionUnitWrapper.getQuestionScore(), SCORE_INTERVAL, order,
									defaultQuestionStructureWrapper.getQuestionId());
							markItems.add(markItem);
						}
						subNumber++;
					}
				}
				mainNumber++;
			}
		}
		SaveMarkItemReq req = new SaveMarkItemReq();
		req.setWorkId(workId);
		req.setMarkItemBeanList(markItems);
		markItemCloudService.saveMarkWork(req);
	}
}
