package cn.com.qmth.examcloud.task.service.consumer;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.task.base.multithread.Consumer;
import cn.com.qmth.examcloud.task.dao.entity.ReportsComputeEntity;
import cn.com.qmth.examcloud.task.service.ReportsComputeService;
import cn.com.qmth.examcloud.task.service.exception.ReportsComputeStopException;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

public class ReportsComputeConsumer extends Consumer<ReportsComputeEntity> {

	private static final Logger LOG = LoggerFactory.getLogger(ReportsComputeConsumer.class);
	private ReportsComputeService reportsComputeService = SpringContextHolder.getBean(ReportsComputeService.class);

	@Override
	public void consume(ReportsComputeEntity et) {
		LOG.info("***************************报表计算开始,projectId:" + et.getProjectId());
		try {
			// 修改报表计算任务状态
			reportsComputeService.updateToComputing(et);
			// 计算报表
			reportsComputeService.compute(et);
			// 修改报表计算任务状态
			reportsComputeService.updateToSuccess(et);
		} catch (ReportsComputeStopException e) {
			// 计算终止
			reportsComputeService.updateToStop(et);
		} catch (StatusException e) {
			// 计算出错
			reportsComputeService.updateToFail(et, e.getDesc());
			LOG.error("***************************报表计算出错,projectId:" + et.getProjectId(), e);
		} catch (Exception e) {
			// 计算出错
			reportsComputeService.updateToFail(et, "系统错误");
			LOG.error("***************************报表计算出错,projectId:" + et.getProjectId(), e);
		} finally {
			// 清除终止标志
			reportsComputeService.clearStopingFlag(et.getId());
		}
		LOG.info("***************************报表计算结束,projectId:" + et.getProjectId());
	}
}
