package cn.com.qmth.examcloud.task.service.job;

import java.util.HashMap;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.marking.api.MarkWorkCloudService;
import cn.com.qmth.examcloud.marking.api.bean.MarkWorkBean;
import cn.com.qmth.examcloud.marking.api.bean.MarkWorkMainBean;
import cn.com.qmth.examcloud.marking.api.request.UpdateMarkWorkReq;
import cn.com.qmth.examcloud.marking.api.response.GetTodoMarkWorkMainResp;
import cn.com.qmth.examcloud.task.service.consumer.MarkWorkCreateConsumer;
import cn.com.qmth.examcloud.task.service.consumer.MarkWorkCreateItemsConsumer;
import cn.com.qmth.examcloud.task.service.producer.MarkWorkCreateItemsProducer;
import cn.com.qmth.examcloud.task.service.producer.MarkWorkCreateProducer;
import cn.com.qmth.examcloud.web.task.AbstractTask;
import cn.com.qmth.examcloud.web.task.ScheduleJob;
import cn.com.qmth.examcloud.web.task.TaskTracker;

@Component("markWorkPaperCreateTask")
public class MarkWorkPaperCreateTask extends AbstractTask {


    @Autowired
    private MarkWorkCloudService markWorkCloudService;


    @Autowired
    TaskTracker TaskTracker;

    @Override
    public void run(ScheduleJob scheduleJob) throws Exception {
        GetTodoMarkWorkMainResp res = markWorkCloudService.getTodoMarkWorkMain();
        MarkWorkMainBean bean = res.getBean();
        if (res.getBean() != null) {
            UpdateMarkWorkReq req = new UpdateMarkWorkReq();
            MarkWorkBean wbean = new MarkWorkBean();
            wbean.setId(bean.getId());
            wbean.setName(bean.getName());
            wbean.setRemark(bean.getRemark());
            wbean.setAppendTime(bean.getAppendTime());
            req.setMarkWorkBean(wbean);
            try {
                for (Long examId : bean.getExamIds()) {
                    createMarkItems(examId, bean.getId());
                    createStudentPapers(examId, bean.getId());
                }
                wbean.setStatus(1);
            } catch (StatusException e) {
                wbean.setStatus(2);
                throw e;
            } catch (Exception e) {
                wbean.setStatus(2);
                throw new StatusException("100001", "评卷工作试卷生成出错 " + e.getMessage(), e);
            } finally {
                markWorkCloudService.updateMarkWork(req);
            }
        }
    }

    private void createMarkItems(Long examId, Long workId) throws InstantiationException, IllegalAccessException{
    	MarkWorkCreateItemsProducer producer = new MarkWorkCreateItemsProducer();
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("examId", examId);
        param.put("workId", workId);
        producer.startDispose(MarkWorkCreateItemsConsumer.class, 10, param);
    }

    private void createStudentPapers(Long examId, Long workId) throws InstantiationException, IllegalAccessException {
        MarkWorkCreateProducer producer = new MarkWorkCreateProducer();
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("examId", examId);
        param.put("workId", workId);
        producer.startDispose(MarkWorkCreateConsumer.class, 10, param);
    }

    @Override
    public TaskTracker getTaskTracker() {
        return TaskTracker;
    }
}
