package cn.com.qmth.examcloud.task.service.producer;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import cn.com.qmth.examcloud.core.oe.admin.api.ExamRecordForMarkingCloudService;
import cn.com.qmth.examcloud.core.oe.admin.api.bean.ExamRecordForMarkingBean;
import cn.com.qmth.examcloud.core.oe.admin.api.request.FindExamRecordForMarkingInfoReq;
import cn.com.qmth.examcloud.core.oe.admin.api.response.FindExamRecordForMarkingInfoResp;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.SysPropertyCacheBean;
import cn.com.qmth.examcloud.task.base.multithread.Producer;
import cn.com.qmth.examcloud.task.service.dto.MarkWorkCreateItemsDto;
import cn.com.qmth.examcloud.web.support.SpringContextHolder;

public class MarkWorkCreateItemsProducer extends Producer {

    private ExamRecordForMarkingCloudService examRecordForMarkingCloudService = SpringContextHolder
            .getBean(ExamRecordForMarkingCloudService.class);


    private static final Logger LOG = LoggerFactory.getLogger(MarkWorkCreateItemsProducer.class);

    @Override
    protected void produce(Map<String, Object> param) throws Exception {
        Long examId = (Long) param.get("examId");
        Long workId = (Long) param.get("workId");
        Long batchSize = 50L;
        SysPropertyCacheBean spc = CacheHelper.getSysProperty("marking.rpc.saveMarkItems.batchSize");
        if (spc != null && spc.getValue() != null) {
            batchSize = (Long) spc.getValue();
        }
        
        FindExamRecordForMarkingInfoReq markingReq = new FindExamRecordForMarkingInfoReq();
        markingReq.setExamId(examId);
        markingReq.setBatchNum(workId + "");
        FindExamRecordForMarkingInfoResp markingResp = examRecordForMarkingCloudService
                .findExamRecordForMarkingInfo(markingReq);
        List<ExamRecordForMarkingBean> examRecordForMarkingBeanList = markingResp.getExamRecordForMarkingBeanList();
        if(examRecordForMarkingBeanList!=null&&examRecordForMarkingBeanList.size()>0) {
        	List<String> result=duplicateRemoval(examRecordForMarkingBeanList);
        	LOG.info("创建评卷工作评分项,试卷套数:" + result.size());
        	subListDispose(result, workId, batchSize);
        }else {
        	LOG.info("创建评卷工作评分项,试卷套数:0");
        }
    }


    private List<String> duplicateRemoval(List<ExamRecordForMarkingBean> list){
    	Set<String> set=new HashSet<String>();
    	for(ExamRecordForMarkingBean bean:list) {
    		set.add(bean.getBasePaperId());
    	}
    	List<String> result = new ArrayList<>(set);
    	return result;
    }
    
    private void subListDispose(List<String> list, Long workId, Long batchSize) throws InterruptedException {
        if (list.size() <= batchSize) {
            // 生产需处理的数据
            offer(new MarkWorkCreateItemsDto(list, workId));
        } else {
            int size = list.size();
            int len = batchSize.intValue();
            int count = (size + len - 1) / len;

            for (int i = 0; i < count; i++) {
                List<String> subList = list.subList(i * len,
                        ((i + 1) * len > size ? size : len * (i + 1)));
                // 生产需处理的数据
                offer(new MarkWorkCreateItemsDto(subList, workId));
            }
        }
    }
}
